#! /bin/bash

#######################
# Check Maven is 3.1.1
mvn_version=`mvn -v | grep 'Apache Maven' | cut -d ' ' -f 3`
if [ "$mvn_version" != "3.1.1" ]
then
    echo "Maven 3.1.1 is required"
    exit 1
fi

#######################
# Check arguments
if [ $# -lt 3 ]
then
    echo "Usage $0 <destDir> <ccin2p3BootProperties> <bootProperties>|<srcDir>"
    echo "<destDir>: new OpenIDM home"
    echo "<ccin2p3BootProperties>: file with CCIN2P3 connectors properties"
    echo "<bootProperties>: file with OpenIDM properties. Use this for a new install."
    echo "<srcDir>: old OpenIDM home. Use this for upgrading an existing OpenIDM. OpenIDM properties will copied from <srcDir>/conf/boot/boot.properties"
    exit 1
fi

upgrade_home=`dirname $0`

if [ ! -d "$1" ]
then
    echo "$1 must be a directory"
    exit 1
fi
openidm_home=$1

if [ ! -f "$2" ]
then
    echo "$2 must be a file"
    exit 1
fi
ccin2p3_prop=$2

if [ -d "$3" ]
then
    old_openidm_home="$3"
    openidm_prop=${old_openidm_home}/conf/boot/boot.properties
else
    old_openidm_home=""
    openidm_prop=$3
fi

# Information
echo "***************************"
echo -n "Configuring OpenIDM-3.2.0-SNAPSHOT "
grep SCM-Revision ${openidm_home}/ui/default/admin/META-INF/MANIFEST.MF
echo "***************************"

#######################
# stop OpenIDM
echo "Stopping openIDM"
sh ${openidm_home}/shutdown.sh
while true
do 
    [ -f ${openidm_home}/.openidm.pid ] || break
    echo -n '.'
    sleep 1
done
echo '[OK]'

#######################
# clean felix-cache
echo -n "Cleaning OSGI cache... "
rm -rf ${openidm_home}/felix-cache/*
echo '[OK]'

#######################
# Backup
backup=upgrade-`date +%s`
echo -n "Backing up in ${backup}... "
cp -R ${openidm_home} ${backup}
[ $? -ne 0 ] && echo [Failed] && exit 1
echo '[OK]'

#######################
# Configure properties
echo -n "Configuring properties... "
# removing ccin2p3 properties
grep -q '# CCIN2P3' ${openidm_home}/conf/boot/boot.properties
if [ $? -eq 0 ]
then
    sed -i -e '/# CCIN2P3.*$/,$d' ${openidm_home}/conf/boot/boot.properties
fi
# replacing OpenIDM properties
if [ -n "${openidm_prop}" -a -f "${openidm_prop}" ]
then
    grep -v '^#' ${openidm_prop} | while read new_prop
    do
        if [ "${new_prop}" != "" ]
        then
            key=`echo ${new_prop} | cut -d '=' -f1`
            sed -i -e "s!^${key}=.*!${new_prop}!" ${openidm_home}/conf/boot/boot.properties
        fi
    done
fi
# adding ccin2p3 properties
echo "" >> ${openidm_home}/conf/boot/boot.properties
cat ${ccin2p3_prop} >> ${openidm_home}/conf/boot/boot.properties
chmod 600 ${openidm_home}/conf/boot/boot.properties

# Get all variables from boot.properties
TEMPFILE=$(mktemp)
grep -v '^#' ${openidm_home}/conf/boot/boot.properties | while read line
do
    if [ "$line" != "" ]
    then
        arg=`echo $line|cut -d '=' -f1|sed 's/\./_/g'`
        val=`echo $line|cut -d '=' -f2-`
        echo $arg=\"$val\" >> $TEMPFILE
    fi
done
source $TEMPFILE
[ $? -ne 0 ] && echo [Failed] && exit 1
rm $TEMPFILE

instance=${iam_repo_database}

# Check if non-production instance has emails routing
if [ "${instance}" != "openidm" ] ; then
	redirect="virtual_alias_maps = pcre:/opt/iam/openidm/conf/postfix.redirect"
	grep -q "$redirect" /etc/postfix/main.cf > /dev/null 2>&1
	if [ $? -ne 0 ] ; then
    	echo '[Failed]'
    	echo 'your instance is not a PRODUCTION instance, and therefore must redirect emails.'
    	echo 'Connect as root and set virtual_alias_maps as follows:'
    	echo "  $redirect"
    	exit 1
    fi
fi

# Install the crontab
if [ -f /bin/systemctl ]
then
cat <<END | crontab -
MAILTO=lionel.schwarz@in2p3.fr
28 * * * * /bin/systemctl status ntpd.service | grep 'Active:' | grep -v running
END
[ $? -ne 0 ] && echo [Failed]
fi

#######################
# Disable console to allow service to run in background
echo -n "Enabling background running... "
sed -i -e 's/, java.util.logging.ConsoleHandler//' \
	-e 's/java.util.logging.ConsoleHandler.level = WARNING/java.util.logging.ConsoleHandler.level = OFF/' \
	-e 's/^java.util.logging.ConsoleHandler.filter/#java.util.logging.ConsoleHandler.filter/' \
    -e 's/^java.util.logging.ConsoleHandler.formatter/#java.util.logging.ConsoleHandler.formatter/' \
	${openidm_home}/conf/logging.properties
[ $? -ne 0 ] && echo [Failed] && exit 1
rm -rf ${openidm_home}/bundle/org.apache.felix.shell.tui-*.jar
echo '[OK]'

#######################
# MySQL
echo -n "Modifying OpenIDM for MySQL backend... "

# Remove OrientDB config
rm -rf ${openidm_home}/conf/repo.orientdb.json

# download MySQL jar
mvn -q dependency:get -Dtransitive=false -DgroupId=mysql -DartifactId=mysql-connector-java -Dversion=5.1.33 \
    -Dpackaging=jar -Ddest=${openidm_home}/bundle/ -DrepoUrl=http://maven.apache.org
[ $? -ne 0 ] && echo [Failed] && exit 1

# modify the SQL script
sed -i -e 's/`openidm`/`'"${iam_repo_database}"'`/' \
        -e 's/^GRANT ALL/-- GRANT ALL/' \
        -e 's/^CREATE SCHEMA/-- CREATE SCHEMA/' \
        ${openidm_home}/db/mysql/scripts/openidm.sql
[ $? -ne 0 ] && echo [Failed] && exit 1
chmod 600 ${openidm_home}/db/mysql/scripts/openidm.sql

# install the custom SQL script
sed -e 's/`openidm`/`'"${iam_repo_database}"'`/' ${upgrade_home}/openidm.custom.sql > ${openidm_home}/db/mysql/scripts/openidm.custom.sql
[ $? -ne 0 ] && echo [Failed] && exit 1
chmod 600 ${openidm_home}/db/mysql/scripts/openidm.custom.sql

# patching repo.jdbc.json
python ${upgrade_home}/mergeConfig.py ${openidm_home}/db/mysql/conf/repo.jdbc.json ${upgrade_home}/conf/patch/repo.jdbc.json > /tmp/repo.jdbc.json
[ $? -ne 0 ] && echo [Failed] && exit 1
mv /tmp/repo.jdbc.json ${openidm_home}/conf/repo.jdbc.json 

echo '[OK]'

#######################
# Config
echo -n "Installing CC configuration... "
echo -n "[JSON files] "
cp -p ${upgrade_home}/conf/*.json ${openidm_home}/conf
if [ -d ${upgrade_home}/conf/${instance} ] ; then
	if [ -f ${openidm_home}/conf/postfix.redirect ] ; then
		chmod 600 ${openidm_home}/conf/postfix.redirect
	fi
	cp -p ${upgrade_home}/conf/${instance}/* ${openidm_home}/conf
	if [ -f ${openidm_home}/conf/postfix.redirect ] ; then
		chmod 400 ${openidm_home}/conf/postfix.redirect
	fi
fi
echo -n "[scripts] "
cp -p ${upgrade_home}/script/* ${openidm_home}/script
echo -n "[Patching access.js] "
# Update script/access.js
sed -i -e '/httpAccessConfig.configs.unshift/,$d' ${openidm_home}/script/access.js
for cf in `ls ${upgrade_home}/access/*.json`
do
	content=`cat ${cf}`
	cat <<END >> ${openidm_home}/script/access.js
httpAccessConfig.configs.unshift(
${content}
);
END
done

# Patching managed.json
echo -n "[Patching managed.json] "
# 1. add custom config
python ${upgrade_home}/mergeConfig.py ${openidm_home}/conf/managed.json ${upgrade_home}/conf/patch/managed.json > /tmp/managed.json
[ $? -ne 0 ] && echo [Failed] && exit 1
# 2. change alias name for encryption
seckey=${openidm_config_crypto_alias}
sed -e "s/openidm-sym-default/${seckey}/g" /tmp/managed.json > ${openidm_home}/conf/managed.json
[ $? -ne 0 ] && echo [Failed] && exit 1

for f in policy audit ui-configuration authentication
do
	echo -n "[Patching ${f}.json] "
	python ${upgrade_home}/mergeConfig.py ${openidm_home}/conf/${f}.json ${upgrade_home}/conf/patch/${f}.json > /tmp/${f}.json
	[ $? -ne 0 ] && echo [Failed] && exit 1
	mv /tmp/${f}.json ${openidm_home}/conf/${f}.json
done

echo '[OK]'

#######################
# SSL HTTPS
echo "Configuring SSL/HTTPS... "
if [ "${old_openidm_home}" == "" ]
then
    echo "* ${openidm_keystore_location} "
    # Test if alias is not the default: 'openidm-localhost'
    alias=${openidm_https_keystore_cert_alias}
    if [ "${alias}" != "openidm-localhost" ]
    then
        # test if the keystore is empty with the default password 'changeit' (0 entries or 0 entrées)
        keytool -list -keystore ${openidm_home}/${openidm_keystore_location} -storetype JCEKS -storepass changeit | grep -q '0 entr'
        if [ $? -eq 0 ]
        then
            # PEM -> PKCS12
            openssl pkcs12 -export -out /tmp/${alias}.p12 -name ${alias} -inkey ${alias}.key -in ${alias}.pem -passout pass:dummypwd
            # PKCS12 -> keystore
            keytool -noprompt -importkeystore -srckeystore /tmp/${alias}.p12 -srcstoretype pkcs12 -srcstorepass dummypwd \
                    -destkeystore ${openidm_home}/${openidm_keystore_location} -deststoretype JCEKS -deststorepass changeit
            rm /tmp/${alias}.p12
        fi
    fi
    echo "* ${openidm_truststore_location} "
    rm -rf ${openidm_home}/${openidm_truststore_location}
    for ca in CNRS2 CNRS2-Standard
    do
        curl --insecure --silent -o ${openidm_home}/security/${ca}.pem \
            'https://igc.services.cnrs.fr/search_CA_certificate/?CA='"${ca}&view_CA=${ca}"'&lang=fr&act=save&body=view_ca.html'
        openssl x509 -in ${openidm_home}/security/${ca}.pem | keytool -keystore ${openidm_home}/${openidm_truststore_location} \
            -storepass ${openidm_truststore_password} -import -alias ${ca} -noprompt > /dev/null
    done
else
    echo "* ${openidm_keystore_location} "
    cp -p ${old_openidm_home}/${openidm_keystore_location} ${openidm_home}/${openidm_keystore_location}
    echo "* ${openidm_truststore_location} "
    cp -p ${old_openidm_home}/${openidm_truststore_location} ${openidm_home}/${openidm_truststore_location}
    if [ -f ${old_openidm_home}/${krb5_keytab} ]
    then
        echo "* ${krb5_keytab} "
        cp -p ${old_openidm_home}/${krb5_keytab} ${openidm_home}/${krb5_keytab}
    fi
fi


echo '[OK]'

#######################
# Connectors
echo -n "Configuring LDAP connector... "
python ${upgrade_home}/mergeConfig.py \
    ${openidm_home}/samples/sample2/conf/provisioner.openicf-ldap.json \
    ${upgrade_home}/conf/patch/provisioner.openicf-ldap.json \
    > ${openidm_home}/conf/provisioner.openicf-ldap.json
[ $? -ne 0 ] && echo [Failed] && exit 1
chmod 600 ${openidm_home}/conf/provisioner.openicf-ldap.json
echo '[OK]'

echo -n "Installing CC connectors... "
# Third-party libs are installed in /lib 
# Do not clean /lib, as it contains already installed libs

# Download parent: needed for copy-dependencies
# TODO FIXME: version is hardcoded
mvn -q dependency:get -Dtransitive=true -DremoteRepositories=CCIN2P3::::http://maven.in2p3.fr -DgroupId=fr.in2p3.openicf.connectors \
   	-DartifactId=connectors -Dversion=1.1.1.1 -Dpackaging=pom -Ddest=./pom.xml
[ $? -ne 0 ] && echo [Failed] && exit 1

# each connector
for connector in ${cc_connectors}
do
	connectorVariable="${connector}_connectorBundle"
	connectorArtifact="${!connectorVariable}"
	[ -z "${connectorArtifact}" ] && echo "[Failed] Missing ${connectorVariable}" && exit 1
	echo -n " ${connectorArtifact} "
    # download and install the JAR
	# TODO -Ddest is deprecated , must remove it and do a copy-dependency from local repo to ${openidm_home}/connectors
    mvn -q dependency:get -Dtransitive=false -DremoteRepositories=CCIN2P3::::http://maven.in2p3.fr \
    						-Dartifact=${connectorArtifact}:jar \
                            -Ddest=${openidm_home}/connectors
    [ $? -ne 0 ] && echo [Failed] && exit 1
    # download the POM into the ${connector} directory
	mkdir ${connector}
    mvn -q dependency:get -Dtransitive=false -DremoteRepositories=CCIN2P3::::http://maven.in2p3.fr \
    						-Dartifact=${connectorArtifact}:pom \
                            -Ddest=${connector}/pom.xml
    [ $? -ne 0 ] && echo [Failed] && exit 1
	
    # download and install dependencies
	# outputDirectory is RELATIVE to pom.xml !!!
    mvn -q -f ${connector}/pom.xml dependency:copy-dependencies -DoutputDirectory=../${openidm_home}/lib/ -DincludeScope=runtime \
                                        -DexcludeArtifactIds=connector-framework -Dtype=jar
    [ $? -ne 0 ] && echo [Failed] && exit 1
    rm -rf ${connector}
    
    # extract the configuration file to /conf
	name=`echo ${connectorArtifact} | cut -d ':' -f '2,3' | sed -e 's/:/-/'` # fr.in2p3.connectors:udb-connector:1.1 => udb-connector-1.1
	unzip -q -o -d ${openidm_home}/conf/ ${openidm_home}/connectors/${name}.jar "provisioner.openicf-*.json"	
done
rm -rf pom.xml
echo '[OK]'
    
#######################
# Workflows
echo -n "Installing CC workflows... "
rm -rf ${openidm_home}/workflow/*
for workflow in ${cc_workflows}
do
	workflowVariable="${workflow}_artifact"
	workflowArtifact="${!workflowVariable}"
    echo -n " $workflowArtifact "
    mvn -q dependency:get -Dtransitive=false -DremoteRepositories=CCIN2P3::::http://maven.in2p3.fr -Dartifact=${workflowArtifact}:bar -Ddest=${openidm_home}/workflow/
    [ $? -ne 0 ] && echo [Failed] && exit 1
    rm -rf ${openidm_home}/bundle/${workflow}-*.jar
    mvn -q dependency:get -Dtransitive=false -DremoteRepositories=CCIN2P3::::http://maven.in2p3.fr -Dartifact=${workflowArtifact}:jar -Ddest=${openidm_home}/bundle/
    [ $? -ne 0 ] && echo [Failed] && exit 1
done    
echo '[OK]'

# UI
echo -n "Customizing UI... "
# From r5497, path is 'ui/default/selfservice' instead of 'ui/default/enduser'
selfservice="selfservice"
[ -d ${openidm_home}/ui/default/enduser ] && selfservice="enduser"

cp -pr ${upgrade_home}/ui/default/enduser/* ${openidm_home}/ui/default/${selfservice}
rm -rf ${openidm_home}/ui/extension/*
mkdir -p ${openidm_home}/ui/extension
cp -pr ${upgrade_home}/ui/extension/* ${openidm_home}/ui/extension
ln -s ${instance}.png ${openidm_home}/ui/extension/images/logo.png
ln -s ${instance}.png ${openidm_home}/ui/extension/images/login-logo.png
# allow Users tab for master.logon
# TODO: FIXME does not work when ${openidm_home} was already installed
# awk: when "var uiRoles =": marker (f) =1, print file, no not print lines until meets "};": marker (f) = 0
awk '/var uiRoles = /{f=1;while (getline < "'${upgrade_home}/conf/uiRoles.js'"){print}} {if(!f)print}/};/{f=0}' \
	${openidm_home}/ui/default/${selfservice}/public/org/forgerock/openidm/ui/common/delegates/AuthnDelegate.js > /tmp/AuthnDelegate.js
[ $? -ne 0 ] && echo [Failed] && exit 1
mv /tmp/AuthnDelegate.js ${openidm_home}/ui/default/${selfservice}/public/org/forgerock/openidm/ui/common/delegates/AuthnDelegate.js

# patch the en/translations.json for UI labels
python ${upgrade_home}/mergeConfig.py \
    ${openidm_home}/ui/default/${selfservice}/public/locales/en/translation.json \
    ${upgrade_home}/ui/patch/enduser/public/locales/en/translation.json > /tmp/translations.json
[ $? -ne 0 ] && echo [Failed] && exit 1
mv /tmp/translations.json ${openidm_home}/ui/default/${selfservice}/public/locales/en/translation.json

# TODO patch default/enduser/public/org/forgerock/openidm/ui/admin/users/AdminUserProfileView.js
echo '[OK]'

#######################
# Validating config
# This cannot work on Jenkins slave as cli.sh calls 'java' and we do not know what is 'java' on slaves...
#${openidm_home}/cli.sh validate

echo "****"

#######################
cat <<END
You have nearly finished configuring OpenIDM.
Still some security stuff to setup:
- make sure you have the Kerberos keytab installed at ${openidm_home}/${krb5_keytab} (the Kerberos keytab is needed by REMCTL based connectors)
- change connectors passwords in ${openidm_home}/conf/boot/boot.properties
END

if [ "${openidm_keystore_password}" == "changeit" ]
then
    echo "- PLEASE change the keystore password in ${openidm_home}/conf/boot/boot.properties"
fi

if [[ ! "${openidm_keystore_password}" =~ (OBF|CRYPT):.* ]]
then
cat <<END
- You should encrypt the keystore password. Run :
  $ java -jar ${openidm_home}/bundle/openidm-crypto-*.jar <PLAIN_PASSWORD>
  and change ${openidm_home}/conf/boot/boot.properties.
See 'https://forge.in2p3.fr/projects/openidm/wiki/Configuration_de_s%C3%A9curit%C3%A9#Modifier-et-encrypter-le-mot-de-passe-du-keystore'
END
fi

echo

if [ "${old_openidm_home}" == "" ]
then
	cat <<END
If you need to re-create the MySQL database:
- run:
mysql --host=${iam_repo_host} --port=${iam_repo_port} --user=${iam_repo_user} --password < ${openidm_home}/db/mysql/scripts/openidm.sql
- change passwords for internal users in ${openidm_home}/db/mysql/scripts/openidm.custom.sql
- run:
mysql --host=${iam_repo_host} --port=${iam_repo_port} --user=${iam_repo_user} --password < ${openidm_home}/db/mysql/scripts/openidm.custom.sql
END
else
	diff -q ${old_openidm_home}/db/mysql/scripts/openidm.sql ${openidm_home}/db/mysql/scripts/openidm.sql
	if [ $? -eq 1 ] ; then
		echo "CAUTION: database schema has changed. here are the differences:"
		diff ${old_openidm_home}/db/mysql/scripts/openidm.sql ${openidm_home}/db/mysql/scripts/openidm.sql
	fi
fi
echo

cat <<END
ADDITIONAL things to do:
- (Display organization on User Profile) : Merge ${openidm_home}/ui/default/${selfservice}/public/org/forgerock/openidm/ui/admin/users/AdminUserProfileView.js
with ${upgrade_home}/ui/patch/enduser/public/org/forgerock/openidm/ui/admin/users/AdminUserProfileView.js
- (Display user name on task title) : Merge ${openidm_home}/ui/default/${selfservice}/public/org/forgerock/openidm/ui/admin/workflow/tasks/TasksMenuView.js
with ${upgrade_home}/ui/patch/enduser/public/org/forgerock/openidm/ui/admin/workflow/tasks/TasksMenuView.js
END
